<?php

/**
 * @param $request HTTP request
 * @throws Api_Error_Response
 * @return Api_Response
 */
function generate_nachos_response($request) {
    $nachos = new Nachos();
    return $nachos->handle_request($request);
}

/**
 * Controller for the HTTP API to manage /api/nachos
 */
class Nachos {
    /**
     * Main entry point for requests that should be handled by nachos
     *
     * @param Api_Request $request The HTTP request we're responding to
     * @throws Api_Error_Response
     * @returns Api_Response
     */
    public function handle_request(Api_Request $request) {
        $endpoint = op5config::instance()->getConfig('http_api.nachos.endpoint');

        if ($endpoint == null) {
            $config_path = op5config::instance()->getPathForNamespace('http_api');
            throw new Api_Error_Response('Nachos endpoint not set', 500,
                "Nachos endpoint is not set. Make sure the endpoint is set in $config_path");
        }

        if($request->get_method() == 'POST') {
            $body = $request->get_body();

            if (!array_key_exists('export_type', $body)) {
                throw new Api_Error_Response('Bad request', 400, "export_type must be provided");
            }

            $export_type = $body['export_type'];

            if ($export_type !== "full_export" && $export_type !== "user_export") {
                throw new Api_Error_Response('Bad request', 400, "export_type must be either 'full_export' or 'user_export'");
            }

            $data = json_encode(array(
                'user' => $request->get_username(),
                'export_type' => $export_type
            ));

            $endpoint = $endpoint . 'exports';
        } else if ($request->get_method() == 'GET') {
            $data = null;
            $endpoint = $endpoint . 'latestexport';
        } else {
            throw new Api_Error_Response('Method not allowed', 405, "{$request->get_method()} is not a valid method");
        }

        $nachos = $this->query_nachos($endpoint, $data);

        if ($nachos['http_code'] == 0 || $nachos == null) {
            op5log::instance('http_api')->log('debug', "Unable to query nachos. Got return code {$nachos['http_code']}");
            throw new Api_Error_Response('Internal Server Error', 500, "Unable to query Nachos. Is the Nachos service running?");
        }

        return new Api_Response(json_decode($nachos['response']), $nachos['http_code']);
    }

    /**
     * The part that actually communicates with nachos
     *
     * @param $endpoint Nachos API endpoint
     * @param $data json encoded array with the post body for Nachos
     * @throws Api_Error_Response
     * @returns Array
     */
    protected function query_nachos($endpoint, $data) {
        op5log::instance('http_api')->log('debug', "Sending request to nachos endpoint '$endpoint'");

        $handle = curl_init($endpoint);

        if (!empty($data)) {
            curl_setopt($handle, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($handle, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($handle, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
        curl_setopt($handle, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($handle);
        $http_code = curl_getinfo($handle, CURLINFO_HTTP_CODE);

        curl_close($handle);

        op5log::instance('http_api')->log('debug', "Nachos responded with '$response' and http_code '$http_code'");

        return array(
            'response' => $response,
            'http_code' => $http_code
        );
    }

}
